echo "By downloading and using the cuSPARSELt conda package, you accept the terms"    >> "$PREFIX/.messages.txt"
echo "and conditions of the NVIDIA cuSPARSELt Software License Agreement:"            >> "$PREFIX/.messages.txt"
echo "https://docs.nvidia.com/cuda/cusparselt/license.html"                           >> "$PREFIX/.messages.txt"